@@ -74,8 +74,8 @@
 
 # Make version info available
 __ver_major__ = 1
-__ver_minor__ = 3
-__ver_patch__ = 1
+__ver_minor__ = 4
+__ver_patch__ = 2
 __ver_sub__ = ""
 __version__ = "%d.%d.%d%s" % (__ver_major__,__ver_minor__,
                               __ver_patch__,__ver_sub__)
@@ -89,9 +89,11 @@ class Error(Exception):
 
 import _enchant as _e
 import utils
-import unittest
+from pypwl import PyPWL
 
 import os
+import unittest
+import warnings
 
 class DictNotFoundError(Error):
     """Exception raised when a requested dictionary could not be found."""
@@ -141,7 +143,7 @@ class _EnchantObject(object):
     can be called to ensure that this point is not None, raising an
     exception if it is.
     """
-    
+
     def __init__(self):
         """_EnchantObject constructor."""
         self._this = None
@@ -204,11 +206,8 @@ class Broker(_EnchantObject):
 
     def __del__(self):
         """Broker object destructor."""
-        # Calling _free() might fail if python is shutting down
-        try:
-            self._free()
-        except AttributeError:
-            pass
+        if _e is not None:
+          self._free()
             
     def _raise_error(self,default="Unspecified Error",eclass=Error):
         """Overrides _EnchantObject._raise_error to check broker errors."""
@@ -434,7 +433,7 @@ class Broker(_EnchantObject):
             desc = desc.decode("utf-8")
             file = file.decode("utf-8")
             cb_result.append((tag,name,desc,file))
-        # Actual call describer function
+        # Actually call describer function
         _e.enchant_dict_describe_py(dict_data,cb_func)
         return cb_result[0]
         
@@ -452,8 +451,9 @@ class Dict(_EnchantObject):
 
         * check():              check whether a word id spelled correctly
         * suggest():            suggest correct spellings for a word
+        * add():                add a word to the user's personal dictionary
+        * remove():             add a word to the user's personal exclude list
         * add_to_session():     add a word to the current spellcheck session
-        * add_to_pwl():         add a word to the personal dictionary
         * store_replacement():  indicate a replacement for a given word
 
     Information about the dictionary is available using the following
@@ -505,7 +505,7 @@ class Dict(_EnchantObject):
             self._switch_this(broker._request_dict_data(tag),broker)
 
     def __del__(self):
-        """Dict object desstructor."""
+        """Dict object destructor."""
         # Calling free() might fail if python is shutting down
         try:
             self._free()
@@ -598,8 +598,28 @@ class Dict(_EnchantObject):
             return uSuggs
         return suggs
 
+    def add(self,word):
+        """Add a word to the user's personal word list."""
+        self._check_this()
+        if type(word) == unicode:
+            inWord = word.encode("utf-8")
+        else:
+            inWord = word
+        _e.enchant_dict_add(self._this,inWord,len(inWord))
+
+    def remove(self,word):
+        """Add a word to the user's personal exclude list."""
+        self._check_this()
+        if type(word) == unicode:
+            inWord = word.encode("utf-8")
+        else:
+            inWord = word
+        _e.enchant_dict_remove(self._this,inWord,len(inWord))
+
     def add_to_pwl(self,word):
         """Add a word to the user's personal word list."""
+        warnings.warn("Dict.add_to_pwl is deprecated, please use Dict.add",
+                      category=DeprecationWarning)
         self._check_this()
         if type(word) == unicode:
             inWord = word.encode("utf-8")
@@ -608,7 +628,7 @@ class Dict(_EnchantObject):
         _e.enchant_dict_add_to_pwl(self._this,inWord,len(inWord))
 
     def add_to_session(self,word):
-        """Add a word to the session list."""
+        """Add a word to the session personal list."""
         self._check_this()
         if type(word) == unicode:
             inWord = word.encode("utf-8")
@@ -616,6 +636,33 @@ class Dict(_EnchantObject):
             inWord = word
         _e.enchant_dict_add_to_session(self._this,inWord,len(inWord))
 
+    def remove_from_session(self,word):
+        """Add a word to the session exclude list."""
+        self._check_this()
+        if type(word) == unicode:
+            inWord = word.encode("utf-8")
+        else:
+            inWord = word
+        _e.enchant_dict_remove_from_session(self._this,inWord,len(inWord))
+
+    def is_added(self,word):
+        """Check whether a word is in the personal word list."""
+        self._check_this()
+        if type(word) == unicode:
+            inWord = word.encode("utf-8")
+        else:
+            inWord = word
+        return _e.enchant_dict_is_added(self._this,inWord,len(inWord))
+
+    def is_removed(self,word):
+        """Check whether a word is in the personal exclude list."""
+        self._check_this()
+        if type(word) == unicode:
+            inWord = word.encode("utf-8")
+        else:
+            inWord = word
+        return _e.enchant_dict_is_removed(self._this,inWord,len(inWord))
+
     def is_in_session(self,word):
         """Check whether a word is in the session list."""
         self._check_this()
@@ -675,50 +722,81 @@ class Dict(_EnchantObject):
         self.__describe_result = (tag,name,desc,file)
 
 
-
 class DictWithPWL(Dict):
-    """Dictionary with managed personal word list.
+    """Dictionary with separately-managed personal word list.
+
+    NOTE:  As of version 1.4.0, enchant manages a per-user pwl and
+           exclude list.  This class is now only needed if you want
+           to explicitly maintain a separate word list in addition to
+           the default one.
     
     This class behaves as the standard Dict class, but also manages a
     personal word list stored in a seperate file.  The file must be
     specified at creation time by the 'pwl' argument to the constructor.
-    Words added to the dictionary using "add_to_personal" are automatically
-    appended to the pwl file.
+    Words added to the dictionary are automatically appended to the pwl file.
+
+    A personal exclude list can also be managed, by passing another filename
+    to the constructor in the optional 'pel' argument.  If this is not given,
+    requests to exclude words are ignored.
+
+    If either 'pwl' or 'pel' are None, an in-memory word list is used.
+    This will prevent calls to add() and remove() from affecting the user's
+    default word lists.
     
     The Dict object managing the PWL is available as the 'pwl' attribute.
+    The Dict object managing the PEL is available as the 'pel' attribute.
     
     To create a DictWithPWL from the user's default language, use None
     as the 'tag' argument.
     """
     
-    def __init__(self,tag,pwl,broker=None):
+    def __init__(self,tag,pwl=None,pel=None,broker=None):
         """DictWithPWL constructor.
 
-        The argument 'pwl' must be supplied, naming a file containing
-	    the personal word list.  If this file does not exist, it is
-        	created with default permissions.
+        The argument 'pwl', if not None, names a file containing the
+        personal word list.  If this file does not exist, it is created
+       	with default permissions.
+
+        The argument 'pel', if not None, names a file containing the personal
+        exclude list.  If this file does not exist, it is created with
+        default permissions.
         """
-        if pwl is None:
-            raise Error("DictWithPWL must be given a PWL file.")
-        if not os.path.exists(pwl):
-            f = file(pwl,"wt")
-	    f.close()
-	    del f
         Dict.__init__(self,tag,broker)
-        self.pwl = self._broker.request_pwl_dict(pwl)
+        if pwl is not None:
+            if not os.path.exists(pwl):
+                f = file(pwl,"wt")
+	        f.close()
+	        del f
+            self.pwl = self._broker.request_pwl_dict(pwl)
+        else:
+            self.pwl = PyPWL()
+        if pel is not None:
+            if not os.path.exists(pel):
+                f = file(pel,"wt")
+	        f.close()
+	        del f
+            self.pel = self._broker.request_pwl_dict(pel)
+        else:
+            self.pel = PyPWL()
      
     def _check_this(self,msg=None):
        """Extend Dict._check_this() to check PWL validity."""
        if self.pwl is None:
            self._free()
+       if self.pel is None:
+           self._free()
        Dict._check_this(self,msg)
        self.pwl._check_this(msg)
+       self.pel._check_this(msg)
 
     def _free(self):
         """Extend Dict._free() to free the PWL as well."""
         if self.pwl is not None:
             self.pwl._free()
             self.pwl = None
+        if self.pel is not None:
+            self.pel._free()
+            self.pel = None
         Dict._free(self)
         
     def check(self,word):
@@ -728,12 +806,30 @@ class DictWithPWL(Dict):
         True if it is correctly spelled, and false otherwise.  It checks
         both the dictionary and the personal wordlist.
         """
-        if Dict.check(self,word):
-            return True
+        if self.pel.check(word):
+            return False
         if self.pwl.check(word):
             return True
+        if Dict.check(self,word):
+            return True
         return False
 
+    def add(self,word):
+        """Add a word to the associated personal word list.
+        
+        This method adds the given word to the personal word list, and
+        automatically saves the list to disk.
+        """
+        self._check_this()
+        self.pwl.add(word)
+        self.pel.remove(word)
+
+    def remove(self,word):
+        """Add a word to the associated exclude list."""
+        self._check_this()
+        self.pwl.remove(word)
+        self.pel.add(word)
+
     def add_to_pwl(self,word):
         """Add a word to the associated personal word list.
         
@@ -742,6 +838,17 @@ class DictWithPWL(Dict):
         """
         self._check_this()
         self.pwl.add_to_pwl(word)
+        self.pel.remove(word)
+
+    def is_added(self,word):
+        """Check whether a word is in the personal word list."""
+        self._check_this()
+        return self.pwl.is_added(word)
+
+    def is_removed(self,word):
+        """Check whether a word is in the personal exclude list."""
+        self._check_this()
+        return self.pel.is_added(word)
 
 ##  Create a module-level default broker object, and make its important
 ##  methods available at the module level.
@@ -788,6 +895,11 @@ class TestBroker(unittest.TestCase):
         provs = []
         # Find the providers for each language, and a list of all providers
         for (tag,prov) in self.broker.list_dicts():
+            # Skip hyphenation dictionaries installed by OOo
+            if tag.startswith("hyph_") and prov.name == "myspell":
+                continue
+            # Canonicalize separators
+            tag = tag.replace("-","_")
             if langs.has_key(tag):
                 langs[tag].append(prov)
             else:
@@ -800,7 +912,7 @@ class TestBroker(unittest.TestCase):
                 b2 = Broker()
                 b2.set_ordering(tag,prov.name)
                 d = b2.request_dict(tag)
-                self.assertEqual(d.provider,prov)
+                self.assertEqual((d.provider,tag),(prov,tag))
                 del d
                 del b2
         # Place providers that dont have the language in the ordering
@@ -813,7 +925,7 @@ class TestBroker(unittest.TestCase):
                 b2 = Broker()
                 b2.set_ordering(tag,order)
                 d = b2.request_dict(tag)
-                self.assertEqual(d.provider,prov)
+                self.assertEqual((d.provider,tag,order),(prov,tag,order))
                 del d
                 del b2
 
@@ -840,6 +952,9 @@ class TestBroker(unittest.TestCase):
         d2 = self.broker.request_dict("en_US.utf-8")
         self.assert_(self.broker._Broker__live_dicts["en_US"] == 2)
         del d1
+        d1 = self.broker.request_dict(u"en_US")
+        self.assert_(self.broker._Broker__live_dicts["en_US"] == 2)
+        del d1
         del d2
         self.assert_(self.broker._Broker__live_dicts["en_US"] == 0)
 
@@ -847,6 +962,7 @@ class TestBroker(unittest.TestCase):
         """Test that unicode language tags are accepted"""
         d1 = self.broker._request_dict_data(u"en_US")
         self.assert_(d1)
+        _e.enchant_broker_free_dict(self.broker._this,d1)
         d1 = Dict(u"en_US")
         self.assert_(d1)
 
@@ -884,6 +1000,7 @@ class TestDict(unittest.TestCase):
     
     def test_suggest(self):
         """Test that suggest() gets simple suggestions right."""
+        self.assert_(self.dict.check("hello"))
         self.assert_("hello" in self.dict.suggest("helo"))
 
     def test_suggestHang1(self):
@@ -906,8 +1023,32 @@ class TestDict(unittest.TestCase):
         self.failIf(self.dict.is_in_session("Lozz"))
         self.dict.add_to_session("Lozz")
         self.assert_(self.dict.is_in_session("Lozz"))
+        self.assert_(self.dict.is_added("Lozz"))
         self.assert_(self.dict.check("Lozz"))
-            # Check the behavior of default language
+        self.dict.remove_from_session("Lozz")
+        self.failIf(self.dict.check("Lozz"))
+        self.failIf(self.dict.is_in_session("Lozz"))
+        self.dict.remove_from_session("hello")
+        self.failIf(self.dict.check("hello"))
+        self.assert_(self.dict.is_removed("hello"))
+        self.dict.add_to_session("hello")
+
+    def test_AddRemove(self):
+        """Testing adding/removing from default user dictionary."""
+        nonsense = "kxhjsddsi"
+        self.failIf(self.dict.check(nonsense))
+        self.dict.add(nonsense)
+        self.assert_(self.dict.is_added(nonsense))
+        self.assert_(self.dict.check(nonsense))
+        self.dict.remove(nonsense)
+        self.failIf(self.dict.is_added(nonsense))
+        self.failIf(self.dict.check(nonsense))
+        self.dict.remove("pineapple")
+        self.failIf(self.dict.check("pineapple"))
+        self.assert_(self.dict.is_removed("pineapple"))
+        self.failIf(self.dict.is_added("pineapple"))
+        self.dict.add("pineapple")
+        self.assert_(self.dict.check("pineapple"))
     
     def test_DefaultLang(self):
         """Test behavior of default language selection."""
@@ -932,21 +1073,34 @@ class TestPWL(unittest.TestCase):
     """    
     
     def setUp(self):
-        import tempfile
-        try:
-            self.pwlFileNm = tempfile.mkstemp()[1]
-        except (NameError,AttributeError):
-            self.pwlFileNm = tempfile.mktemp()
+        self._tempDir = self._mkdtemp()
+        self._fileName = "pwl.txt"
     
     def tearDown(self):
+        import shutil
+        shutil.rmtree(self._tempDir)
+
+    def _mkdtemp(self):
+        """Backwards-compatability wrapper for tempfile.mkdtemp"""
+        import tempfile
         try:
-            os.remove(self.pwlFileNm)
-        except:
-            pass
-    
+            return tempfile.mkdtemp()
+        except (NameError,AttributeError):
+            nm = tempfile.mktemp()
+            os.mkdir(nm)
+            return nm
+
+    def _path(self,nm=None):
+        if nm is None:
+          nm = self._fileName
+        nm = os.path.join(self._tempDir,nm)
+        if not os.path.exists(nm):
+          file(nm,'w').close()
+        return nm
+
     def setPWLContents(self,contents):
         """Set the contents of the PWL file."""
-        pwlFile = file(self.pwlFileNm,"w")
+        pwlFile = file(self._path(),"w")
         for ln in contents:
             pwlFile.write(ln)
             pwlFile.write("\n")
@@ -954,61 +1108,172 @@ class TestPWL(unittest.TestCase):
         
     def getPWLContents(self):
         """Retreive the contents of the PWL file."""
-        pwlFile = file(self.pwlFileNm,"r")
+        pwlFile = file(self._path(),"r")
         contents = pwlFile.readlines()
         pwlFile.close()
-        return contents
+        return [c.strip() for c in contents]
     
     def test_check(self):
         """Test that basic checking works for PWLs."""
         self.setPWLContents(["Sazz","Lozz"])
-        d = request_pwl_dict(self.pwlFileNm)
+        d = request_pwl_dict(self._path())
         self.assert_(d.check("Sazz"))
         self.assert_(d.check("Lozz"))
         self.failIf(d.check("hello"))
 
     def test_UnicodeFN(self):
         """Test that unicode PWL filenames are accepted."""
-        d = request_pwl_dict(unicode(self.pwlFileNm))
+        d = request_pwl_dict(unicode(self._path()))
         self.assert_(d)
-    
+
     def test_add(self):
         """Test that adding words to a PWL works correctly."""
-        d = request_pwl_dict(self.pwlFileNm)
+        d = request_pwl_dict(self._path())
         self.failIf(d.check("Flagen"))
-        d.add_to_pwl("Flagen")
-        self.assert_(d.check("Flagen"))
-        self.assert_("Flagen\n" in self.getPWLContents())
+        d.add("Esquilax")
+        d.add("Esquilam")
+        self.assert_(d.check("Esquilax"))
+        self.assert_("Esquilax" in self.getPWLContents())
+        self.assert_(d.is_added("Esquilax"))
         
     def test_suggestions(self):
         """Test getting suggestions from a PWL."""
         self.setPWLContents(["Sazz","Lozz"])
-        d = request_pwl_dict(self.pwlFileNm)
-        self.assert_("sazz" in d.suggest("Saz"))
-        self.assert_("lozz" in d.suggest("Saz"))
-        d.add_to_pwl("Flagen")
-        self.assert_("flagen" in d.suggest("Flags"))
+        d = request_pwl_dict(self._path())
+        self.assert_("Sazz" in d.suggest("Saz"))
+        self.assert_("Lozz" in d.suggest("laz"))
+        self.assert_("Sazz" in d.suggest("laz"))
+        d.add("Flagen")
+        self.assert_("Flagen" in d.suggest("Flags"))
         self.failIf("sazz" in d.suggest("Flags"))
     
     def test_DWPWL(self):
         """Test functionality of DictWithPWL."""
         self.setPWLContents(["Sazz","Lozz"])
-        d = DictWithPWL("en_US",self.pwlFileNm)
+        d = DictWithPWL("en_US",self._path(),self._path("pel.txt"))
         self.assert_(d.check("Sazz"))
         self.assert_(d.check("Lozz"))
         self.assert_(d.check("hello"))
         self.failIf(d.check("helo"))
         self.failIf(d.check("Flagen"))
-        d.add_to_pwl("Flagen")
+        d.add("Flagen")
         self.assert_(d.check("Flagen"))
-        self.assert_("Flagen\n" in self.getPWLContents())
+        self.assert_("Flagen" in self.getPWLContents())
+        d.remove("Lozz")
+        d.remove("hello")
+        self.failIf(d.check("Lozz"))
+        self.failIf(d.check("hello"))
+
+    def test_DWPEL(self):
+        """Test functionality of DictWithPWL using exclude list."""
+        self.setPWLContents(["Sazz","Lozz"])
+        d = DictWithPWL("en_US",self._path())
+        self.assert_(d.check("Sazz"))
+        self.assert_(d.check("Lozz"))
+        self.assert_(d.check("hello"))
+        self.failIf(d.check("helo"))
+        self.failIf(d.check("Flagen"))
+        d.add("Flagen")
+        self.assert_(d.check("Flagen"))
+        self.assert_("Flagen" in self.getPWLContents())
+        d.remove("Lozz")
+        self.failIf(d.check("Lozz"))
+
+    def test_DWPWL_empty(self):
+        """Test functionality of DictWithPWL using transient dicts."""
+        d = DictWithPWL("en_US",None,None)
+        self.assert_(d.check("hello"))
+        self.failIf(d.check("helo"))
+        self.failIf(d.check("Flagen"))
+        d.add("Flagen")
+        self.assert_(d.check("Flagen"))
+        d.remove("hello")
+        self.failIf(d.check("hello"))
+        d.add("hello")
+        self.assert_(d.check("hello"))
+
+    def test_PyPWL(self):
+        d = PyPWL()
+        self.assert_(list(d._words) == [])
+        d.add("hello")
+        d.add("there")
+        d.add("duck")
+        ws = list(d._words)
+        self.assert_(len(ws) == 3)
+        self.assert_("hello" in ws)
+        self.assert_("there" in ws)
+        self.assert_("duck" in ws)
+        d.remove("duck")
+        d.remove("notinthere")
+        ws = list(d._words)
+        self.assert_(len(ws) == 2)
+        self.assert_("hello" in ws)
+        self.assert_("there" in ws)
+
+    def test_UnicodeCharsInPath(self):
+        """Test that unicode chars in PWL paths are accepted."""
+        self._fileName = u'test_\xe5\xe4\xf6_ing'
+        d = request_pwl_dict(self._path())
+        self.assert_(d)
+
+
+class TestInstallEnv(unittest.TestCase):
+    """Run all testcases in a variety of install environments."""
+   
+    def setUp(self):
+        self._tempDir = self._mkdtemp()
+        self._insDir = "build"
+    
+    def tearDown(self):
+        import shutil
+        shutil.rmtree(self._tempDir)
+
+    def _mkdtemp(self):
+        """Backwards-compatability wrapper for tempfile.mkdtemp"""
+        import tempfile
+        try:
+            return tempfile.mkdtemp()
+        except (NameError,AttributeError):
+            nm = tempfile.mktemp()
+            os.mkdir(nm)
+            return nm
+
+    def install(self):
+        import os, sys, shutil
+        insdir = os.path.join(self._tempDir,self._insDir)
+        os.makedirs(insdir)
+        shutil.copytree("enchant",os.path.join(insdir,"enchant"))
+
+    def runtests(self):
+        import os, sys
+        insdir = os.path.join(self._tempDir,self._insDir)
+        if isinstance(insdir,unicode):
+          insdir = insdir.encode(sys.getfilesystemencoding())
+        os.environ["PYTHONPATH"] = insdir
+        script = os.path.join(insdir,"enchant","__init__.py")
+        res = os.system("\"%s\" %s" % (sys.executable,script,))
+        self.assertEquals(res,0)
+
+    def test_basic(self):
+        """Test proper functioning of TestInstallEnv suite."""
+        self.install()
+        self.runtests()
+
+    def test_UnicodeInstallPath(self):
+        """Test installation in a path containing unicode chars."""
+        self._insDir = u'test_\xe5\xe4\xf6_ing'
+        self.install()
+        self.runtests()
+
     
 
-def testsuite():
+def testsuite(recurse=True):
     from enchant.checker import TestChecker
     from enchant.tokenize import TestTokenization, TestFilters
     from enchant.tokenize.en import TestTokenizeEN
     suite = unittest.TestSuite()
+    if recurse:
+      suite.addTest(unittest.makeSuite(TestInstallEnv))
     suite.addTest(unittest.makeSuite(TestBroker))
     suite.addTest(unittest.makeSuite(TestDict))
     suite.addTest(unittest.makeSuite(TestPWL))
@@ -1019,10 +1284,14 @@ def testsuite():
     return suite
 
 def runtestsuite():
-    unittest.TextTestRunner().run(testsuite())
+    return unittest.TextTestRunner(verbosity=0).run(testsuite(recurse=False))
 
-# Run regression tests when called from comand-line
+# Run unit tests when called from comand-line
 if __name__ == "__main__":
-    runtestsuite()
+    import sys
+    res = runtestsuite()
+    if len(res.errors) > 0 or len(res.failures) > 0:
+        sys.exit(1)
+    sys.exit(0)
 
 
